//! Remote procedure calls
pub use picodata_types::network_call;
pub use picodata_types::RequestArgs;

use crate::traft::error::Error;
use crate::traft::node;
use crate::traft::Result;

pub mod ddl_apply;
pub mod expel;
pub mod join;
pub mod replication;
pub mod sharding;
pub mod snapshot;
pub mod update_instance;

/// Invoke remote procedure call on a Raft leader.
pub async fn network_call_to_leader<R>(request: &R) -> Result<R::Response>
where
    R: RequestArgs,
{
    let node = node::global()?;
    let leader_id = node.status().leader_id.ok_or(Error::LeaderUnknown)?;
    let leader_address = node.storage.peer_addresses.try_get(leader_id)?;
    let resp = network_call(&leader_address, request).await?;
    Ok(resp)
}
